require "defines"
local BlueprintString = require "blueprintstring.blueprintstring"
BlueprintString.COMPRESS_STRINGS = true
BlueprintString.LINE_LENGTH = 120

function init_gui(player, ignoretech)
	if (not player.force.technologies["automated-construction"].researched and not ignoretech) then
		return
	end

	if (not player.gui.top["blueprint-string-button"]) then
		player.gui.top.add{type="button", name="blueprint-string-button", caption={"main-button"}}
	end
end

game.oninit(function()
	for _, player in pairs(game.players) do
		init_gui(player, false)
	end
end)

game.onevent(defines.events.onplayercreated, function(event)
	init_gui(game.players[event.playerindex], false)
end)

game.onevent(defines.events.onresearchfinished, function(event)
	if (event.research == "automated-construction") then
		for _, player in pairs(game.players) do
			init_gui(player, true)
		end
	end
end)

function expand_gui(player)
	local frame = player.gui.left["blueprint-string"];
	if (frame) then
		frame.destroy();
	else
		frame = player.gui.left.add{type="frame", name="blueprint-string"}
		frame.add{type="label", caption={"textbox-caption"}}
		frame.add{type="textfield", name="blueprint-string-text"}
		frame.add{type="button", name="blueprint-string-load", caption={"load-string"}}
		frame.add{type="button", name="blueprint-string-save", caption={"save-all"}}
	end
end

function trim(s)
	return (s:gsub("^%s*(.-)%s*$", "%1"))
end

function filter_blueprints(inventory)
	local blueprints = {}
	if (inventory) then
		local total = inventory.getitemcount("blueprint")
		local count = 0
		local i = 1
		while (count < total) do
			if (inventory[i] and inventory[i].type == "blueprint") then
				blueprints[i] = inventory[i]
				count = count + inventory[i].count
			end
			i = i + 1

			assert(i < 100000, "Infinite loop in filter_blueprints")
		end
	end
	return blueprints
end

function find_empty_blueprint(player)
	for _, stack in pairs(filter_blueprints(player.getinventory(1))) do
		if (not stack.isblueprintsetup()) then
			return stack
		end
	end

	for _, stack in pairs(filter_blueprints(player.getinventory(2))) do
		if (not stack.isblueprintsetup()) then
			return stack
		end
	end

	return nil
end

function load_blueprint(player)
	local blueprint = find_empty_blueprint(player)
	if (not blueprint) then
		player.print({"no-empty-blueprint"})
		return
	end

	local data = trim(player.gui.left["blueprint-string"]["blueprint-string-text"].text)
	player.gui.left["blueprint-string"]["blueprint-string-text"].text = ""

	if (data == "") then
		player.print({"no-string"})
		return
	end

	local blueprint_format = BlueprintString.fromString(data)
	if (not blueprint_format or type(blueprint_format) ~= "table") then
		player.print({"unknown-format"})
		return
	end

	if (not blueprint_format.icons or type(blueprint_format.icons) ~= "table" or #blueprint_format.icons < 1) then
		player.print({"unknown-format"})
		return
	end

	status, result = pcall(blueprint.setblueprintentities, blueprint_format.entities)
	if (not status) then
		player.print({"blueprint-api-error", result})
		blueprint.setblueprintentities({})
		return
	end

	if (blueprint.isblueprintsetup()) then
		status, result = pcall(function() blueprint.blueprinticons = blueprint_format.icons end)
		if (not status) then
			player.print({"blueprint-icon-error", result})
			blueprint.setblueprintentities({})
			return
		end
	end

end

local blueprints_saved = 0
function save_blueprint(player, stack, filename)
	local icons = stack.blueprinticons
	local entities = stack.getblueprintentities()

	local blueprint_format = {entities = entities, icons = icons}
	local data = BlueprintString.toString(blueprint_format)
	if (#game.players > 1 and player.name and player.name ~= "") then
		filename = player.name .. "-" .. filename
	end
	game.makefile("blueprint-string/" .. filename .. ".txt", data)

	blueprints_saved = blueprints_saved + 1
end

function save_blueprints(player)
	blueprints_saved = 0

	for position, stack in pairs(filter_blueprints(player.getinventory(1))) do
		if (stack.isblueprintsetup()) then
			save_blueprint(player, stack, "toolbar-"..position)
		end
	end

	for position, stack in pairs(filter_blueprints(player.getinventory(2))) do
		if (stack.isblueprintsetup()) then
			save_blueprint(player, stack, "inventory-"..position)
		end
	end

	if (blueprints_saved > 0) then
		player.print({"blueprints-saved", blueprints_saved})
	else
		player.print({"blueprints-not-saved"})
	end

end

game.onevent(defines.events.onguiclick, function(event) 
	local player = game.players[event.element.playerindex]
	if (event.element.name == "blueprint-string-load") then
		load_blueprint(player)
	elseif (event.element.name == "blueprint-string-save") then
		save_blueprints(player)
	elseif (event.element.name == "blueprint-string-button") then
		expand_gui(player)
	end
end)

game.onevent(defines.events.onrobotbuiltentity, function(event) 
	local entity = event.createdentity
	if (entity and entity.type == "assembling-machine" and entity.recipe and not entity.recipe.enabled) then
		-- Delete the recipe

		-- I don't know how to do that, how about deleting the whole building
		local params = {name = entity.name, position = entity.position, force = entity.force, direction = entity.direction}
		entity.destroy()
		game.createentity(params)
	end
end)
